@@ -1,6 +1,32 @@
+# Copyright (c) 2005-2007 Canonical
+#
+# AUTHOR:
+# Michael Vogt <mvo@ubuntu.com>
+#
+# This file is part of GDebi
+#
+# GDebi is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; either version 2 of the License, or (at
+# your option) any later version.
+#
+# GDebi is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with GDebi; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+
 import sys
 import os
 import string
+import warnings
+from warnings import warn
+warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
 import apt
 import apt_pkg
 
@@ -17,18 +43,26 @@ import time
 import thread
 import re
 
-from DebPackage import DebPackage, MyCache
+from DebPackage import DebPackage, Cache
 from SimpleGladeApp import SimpleGladeApp
 from apt.progress import InstallProgress
-
+from GDebiCommon import GDebiCommon
 from gettext import gettext as _
 
-def utf8(str):
-  return unicode(str, 'latin1').encode('utf-8')
+# the timeout when the termial is expanded if no activity from dpkg
+# is happening 
+GDEBI_TERMINAL_TIMEOUT=4*60.0
 
-class GDebi(SimpleGladeApp):
+def utf8(str):
+  try:
+    return unicode(str).encode('utf-8')
+  except:
+    return unicode(str, 'latin1').encode('utf-8')
+
+class GDebi(SimpleGladeApp, GDebiCommon):
 
     def __init__(self, datadir, options, file=""):
+        GDebiCommon.__init__(self,datadir,options,file)
         localesApp="gdebi"
         localesDir="/usr/share/locale"
         gtk.glade.bindtextdomain(localesApp, localesDir)
@@ -36,13 +70,16 @@ class GDebi(SimpleGladeApp):
 
         SimpleGladeApp.__init__(self, domain="gdebi",
                                 path=datadir+"/gdebi.glade")
-
         # use a nicer default icon
         icons = gtk.icon_theme_get_default()
-        logo=icons.load_icon("gnome-mime-application-x-deb", 48, 0)
-        if logo != "":
-	        gtk.window_set_default_icon_list(logo)
-
+        try:
+          logo=icons.load_icon("gnome-mime-application-x-deb", 48, 0)
+          if logo != "":
+            gtk.window_set_default_icon_list(logo)
+        except Exception, e:
+          print "Error loading logo"
+          pass
+  
         # set image of button "install"  manually, since it is overriden 
         #by set_label otherwise
         img = gtk.Image()
@@ -66,21 +103,12 @@ class GDebi(SimpleGladeApp):
 
         # show what we have
         self.window_main.show()
-
+        
         self.cprogress = self.CacheProgressAdapter(self.progressbar_cache)
-        self._cache = MyCache(self.cprogress)
-        if self._cache._depcache.BrokenCount > 0:
-                err_header = _("Broken dependencies")
-                err_body = _("Your system has broken dependencies. "
-                             "This application can not continue until "
-                             "this is fixed. "
-                             "To fix it run 'sudo synaptic' or "
-                             "'sudo apt-get install -f' "
-                             "in a terminal window.")
-                self.show_alert(gtk.MESSAGE_ERROR, err_header, err_body)
-                sys.exit(1)
+        if not self.openCache():
+            self.show_alert(gtk.MESSAGE_ERROR, self.error_header, self.error_body)
+            sys.exit(1)
         self.statusbar_main.push(self.context, "")
-        self._options = options
         
         # setup the details treeview
         self.details_list = gtk.ListStore(gobject.TYPE_STRING)
@@ -128,15 +156,9 @@ class GDebi(SimpleGladeApp):
                 self.window_main.window.set_cursor(None)
 
     def open(self, file):
-        try:
-            self._deb = DebPackage(self._cache, file)
-        except (IOError,SystemError),e:
-            err_header = _("Could not open '%s'" % os.path.basename(file))
-            err_body = _("The package might be corrupted or you are not "
-                         "allowed to open the file. Check the permissions "
-                         "of the file.")
-            self.show_alert(gtk.MESSAGE_ERROR, err_header, err_body)
-
+        res = GDebiCommon.open(self, file)
+        if res == False:
+            self.show_alert(gtk.MESSAGE_ERROR, self.error_header, self.error_body)
             return False
             
         self.statusbar_main.push(self.context, "")
@@ -217,6 +239,10 @@ class GDebi(SimpleGladeApp):
             self.button_details.hide()
             return
 
+        # set version_info_{msg,title} strings
+        self.compareDebWithCache()
+        self.getChanges()
+
         if self._deb.compareToVersionInCache() == DebPackage.VERSION_SAME:
             self.label_status.set_text(_("Same version is already installed"))
             self.button_install.set_label(_("_Reinstall Package"))
@@ -225,57 +251,24 @@ class GDebi(SimpleGladeApp):
             self.button_details.hide()
             return
 
-        # check if the package is available in the normal sources as well
-        res = self._deb.compareToVersionInCache(useInstalled=False)
-        if not self._options.non_interactive and res != DebPackage.NO_VERSION:
-            pkg = self._cache[self._deb.pkgName]
-            title = msg = ""
-            
-            # FIXME: make this strs better, improve the dialog by
-            # providing a option to install from repository directly
-            # (when possible)
-            if res == DebPackage.VERSION_SAME:
-                if self._cache.downloadable(pkg,useCandidate=True):
-                    title = _("Same version is available in a software channel")
-                    msg = _("You are recommended to install the software "
-                            "from the channel instead.")
-            elif res == DebPackage.VERSION_IS_NEWER:
-                if self._cache.downloadable(pkg,useCandidate=True):
-                    title = _("An older version is available in a software channel")
-                    msg = _("Generally you are recommended to install "
-                            "the version from the software channel, since "
-                            "it is usually better supported.")
-            elif res == DebPackage.VERSION_OUTDATED:
-                if self._cache.downloadable(pkg,useCandidate=True):
-                    title = _("A later version is available in a software "
-                              "channel")
-                    msg = _("You are strongly advised to install "
-                            "the version from the software channel, since "
-                            "it is usually better supported.")
+        if self.version_info_title != "" and self.version_info_msg != "":
+            msg = "<big><b>%s</b></big>\n\n%s" % (self.version_info_title,
+              self.version_info_msg)
+            dialog = gtk.MessageDialog(parent=self.window_main,
+                                       flags=gtk.DIALOG_MODAL,
+                                       type=gtk.MESSAGE_INFO,
+                                       buttons=gtk.BUTTONS_CLOSE)
+            dialog.set_markup(msg)
+            dialog.run()
+            dialog.destroy()
 
-            if title != "" and msg != "":
-                msg = "<big><b>%s</b></big>\n\n%s" % (title,msg)
-                dialog = gtk.MessageDialog(parent=self.window_main,
-                                           flags=gtk.DIALOG_MODAL,
-                                           type=gtk.MESSAGE_INFO,
-                                           buttons=gtk.BUTTONS_CLOSE)
-                dialog.set_markup(msg)
-                dialog.run()
-                dialog.destroy()
-
-        (install, remove, unauthenticated) = self._deb.requiredChanges
-        deps = ""
-        if len(remove) == len(install) == 0:
-            deps = _("All dependencies are satisfied")
+        # load changes into (self.install, self.remove, self.unauthenticated)
+        if len(self.remove) == len(self.install) == 0:
             self.button_details.hide()
         else:
             self.button_details.show()
-        if len(remove) > 0:
-            # FIXME: use ngettext here
-            deps += _("Requires the <b>removal</b> of %s packages\n") % len(remove)
-        if len(install) > 0:
-            deps += _("Requires the installation of %s packages") % len(install)
-        self.label_status.set_markup(deps)
+            
+        self.label_status.set_markup(self.deps)
         img = gtk.Image()
         img.set_from_stock(gtk.STOCK_APPLY,gtk.ICON_SIZE_BUTTON)
         self.button_install.set_image(img)
@@ -285,11 +278,13 @@ class GDebi(SimpleGladeApp):
 
     def on_button_details_clicked(self, widget):
         #print "on_button_details_clicked"
-        (install, remove, unauthenticated) = self._deb.requiredChanges
+        # sanity check
+        if not self._deb:
+          return
         self.details_list.clear()
-        for rm in remove:
+        for rm in self.remove:
             self.details_list.append([_("<b>To be removed: %s</b>") % rm])
-        for inst in install:
+        for inst in self.install:
             self.details_list.append([_("To be installed: %s") % inst])
         self.dialog_details.set_transient_for(self.window_main)
         self.dialog_details.run()
@@ -326,10 +321,21 @@ class GDebi(SimpleGladeApp):
         self.dialog_about.hide()
 
     def on_button_install_clicked(self, widget):
-        #print "install"
+        # check if we actually have a deb, see #213725
+        if not self._deb:
+            err_header = _("File not found")
+            err_body = _("You tried to install a file that does not "
+                         "(or no longer) exist. ")
+            dia = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR,
+                                    gtk.BUTTONS_OK, "")
+            dia.set_markup("<b><big>%s</big></b>" % err_header)
+            dia.format_secondary_text(err_body)
+            dia.run()
+            dia.destroy()
+            return
+        # do it
         self.statusbar_main.push(self.context,_("Installing package file..."))
-        (install, remove, unauthenticated) = self._deb.requiredChanges
-        if widget != None and len(unauthenticated) > 0:
+        if widget != None and len(self.unauthenticated) > 0:
             primary = _("Install unauthenticated software?")
             secondary = _("Malicous software can damage your data "
                           "and take control of your system.\n\n"
@@ -347,7 +353,7 @@ class GDebi(SimpleGladeApp):
             textview.set_cursor_visible(False)
             textview.set_editable(False) 
             buf = textview.get_buffer()
-            buf.set_text("\n".join(unauthenticated))
+            buf.set_text("\n".join(self.unauthenticated))
             scrolled.add(textview)
             scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
             scrolled.show()
@@ -358,25 +364,25 @@ class GDebi(SimpleGladeApp):
             if res != gtk.RESPONSE_YES:
                 return
 
+        msg_hdr = _("You need to grant administrative rights to install software")
+        msg_bdy = _("""
+It is a possible security risk to install packages files manually.
+Install software from trustworthy software distributors only.
+""")
         if os.getuid() != 0:
             os.execl("/usr/bin/gksu", "gksu", "--desktop",
                      "/usr/share/applications/gdebi.desktop",
+                     "--message","<big><b>%s</b></big>\n\n%s" % (msg_hdr,msg_bdy),
+                     "--always-ask-pass",
                      "--", "gdebi-gtk", "--non-interactive",
                      self._deb.file)
-            return
-
-        # check if we can lock the apt database
-        try:
-            apt_pkg.PkgSystemLock()
-        except SystemError:
-            header = _("Only one software management tool is allowed to"
-                       " run at the same time")
-            body = _("Please close the other application e.g. 'Update "
-                     "Manager', 'aptitude' or 'Synaptic' first.")
-            self.show_alert(gtk.MESSAGE_ERROR, header, body)
-            return
-        apt_pkg.PkgSystemUnLock()
 
+        if not self.try_acquire_lock():
+            self.statusbar_main.push(self.context,
+                                     _("Failed to install package file"))
+            self.show_alert(gtk.MESSAGE_ERROR, self.error_header, self.error_body)
+            return False
+            
         # lock for install
         self.window_main.set_sensitive(False)
         self.button_deb_install_close.set_sensitive(False)
@@ -384,27 +390,17 @@ class GDebi(SimpleGladeApp):
         #self._term.feed(str(0x1b)+"[2J")
         self.dialog_deb_install.set_transient_for(self.window_main)
         self.dialog_deb_install.show_all()
-
-        # install the dependecnies
-        if len(install) > 0 or len(remove) > 0:
-            # check if we can lock the apt database
-            try:
-                apt_pkg.PkgSystemLock()
-            except SystemError:
-                header = _("Only one software management tool is allowed to"
-                           " run at the same time")
-                body = _("Please close the other application e.g. 'Update "
-                         "Manager', 'aptitude' or 'Synaptic' first.")
-                self.show_alert(gtk.MESSAGE_ERROR, header, body)
-                self.dialog_deb_install.hide()
-                self.window_main.set_sensitive(True)
-                return
+        
+        if len(self.install) > 0 or len(self.remove) > 0:
             # FIXME: use the new python-apt acquire interface here,
             # or rather use it in the apt module and raise exception
             # when stuff goes wrong!
+            if not self.acquire_lock():
+              self.show_alert(gtk.MESSAGE_ERROR, self.error_header, self.error_body)
+              return False
             fprogress = self.FetchProgressAdapter(self.progressbar_install,
-                                                  self.label_action,
-                                                  self.dialog_deb_install)
+                                                self.label_action,
+                                                self.dialog_deb_install)
             iprogress = self.InstallProgressAdapter(self.progressbar_install,
                                                     self._term,
                                                     self.label_action,
@@ -417,22 +413,23 @@ class GDebi(SimpleGladeApp):
                 errMsg = "%s" % msg
                 header = _("Could not download all required files")
                 body = _("Please check your internet connection or "
-                         "installation medium.")
+                        "installation medium.")
             except SystemError, msg:
                 res = False
                 header = _("Could not install all dependencies"),
                 body = _("Usually this is related to an error of the "
-                         "software distributor. See the terminal window for "
-                         "more details.")
+                        "software distributor. See the terminal window for "
+                        "more details.")
             if not res:
-                self.show_alert(gtk.MESSAGE_ERROR, header, body, msg)
-                
-                self.label_install_status.set_markup("<span foreground=\"red\" weight=\"bold\">%s</span>" % primary)
+                self.show_alert(gtk.MESSAGE_ERROR, header, body, msg,
+                                parent=self.dialog_deb_install)
+                    
+                self.label_install_status.set_markup("<span foreground=\"red\" weight=\"bold\">%s</span>" % header)
                 self.button_deb_install_close.set_sensitive(True)
                 self.button_deb_install_close.grab_default()
-		self.statusbar_main.push(self.context,_("Failed to install package file"))
+                self.statusbar_main.push(self.context,_("Failed to install package file"))
                 return 
-
+    
         # install the package itself
         self.label_action.set_markup("<b><big>"+_("Installing package file")+"</big></b>")
         dprogress = self.DpkgInstallProgress(self._deb.file,
@@ -456,8 +453,8 @@ class GDebi(SimpleGladeApp):
         #self.window_main.set_property("urgency-hint", 1)
 
         # reopen the cache, reread the file, FIXME: add progress reporting
-        #self._cache = MyCache(self.cprogress)
-        self._cache = MyCache()
+        #self._cache = Cache(self.cprogress)
+        self._cache = Cache()
         if self._cache._depcache.BrokenCount > 0:
             err_header = _("Failed to completely install all dependencies")
             err_body = _("To fix this run 'sudo apt-get install -f' in a "
@@ -487,17 +484,20 @@ class GDebi(SimpleGladeApp):
         return self._term
 
 
-    def show_alert(self, type, header, body=None, details=None):
-        self.dialog_hig.set_transient_for(self.window_main)
+    def show_alert(self, type, header, body=None, details=None, parent=None):
+        if parent is not None:
+             self.dialog_hig.set_transient_for(parent)
+        else:
+             self.dialog_hig.set_transient_for(self.window_main)
 
         message = "<b><big>%s</big></b>" % header
         if not body == None:
-            message = "%s\n\n%s" % (message, body)
+             message = "%s\n\n%s" % (message, body)
         self.label_hig.set_markup(message)
   
         if not details == None:
              buffer = self.textview_hig.get_buffer()
-             buffer.set_text(details)
+             buffer.set_text(str(details))
              self.expander_hig.set_expanded(False)
              self.expander_hig.show()
              
@@ -512,7 +512,7 @@ class GDebi(SimpleGladeApp):
         self.dialog_hig.hide()
         if res == gtk.RESPONSE_CLOSE:
             return True
-
+        return False
         
     # embedded classes
     class DpkgInstallProgress(object):
@@ -521,8 +521,9 @@ class GDebi(SimpleGladeApp):
             self.status = status
             self.progress = progress
             self.term = term
-            self.expander = expander
-            self.expander.set_expanded(False)
+            self.term_expander = expander
+            self.time_last_update = time.time()
+            self.term_expander.set_expanded(False)
         def commit(self):
             def finish_dpkg(term, pid, status, lock):
                 " helper "
@@ -550,16 +551,17 @@ class GDebi(SimpleGladeApp):
             # the command
             cmd = "/usr/bin/dpkg"
             argv = [cmd,"--status-fd", "%s"%writefd, "-i", self.debfile]
-            env = ["VTE_PTY_KEEP_FD=%s"% writefd]
+            env = ["VTE_PTY_KEEP_FD=%s"% writefd,
+                   "DEBIAN_FRONTEND=gnome",
+                   "APT_LISTCHANGES_FRONTEND=gtk"]
             #print cmd
             #print argv
             #print env
             #print self.term
 
-
             # prepare for the fork
             reaper = vte.reaper_get()
-            reaper.connect("child-exited", finish_dpkg, lock)
+            signal_id = reaper.connect("child-exited", finish_dpkg, lock)
             pid = self.term.fork_command(command=cmd, argv=argv, envv=env)
             read = ""
             while lock.locked():
@@ -571,23 +573,28 @@ class GDebi(SimpleGladeApp):
                         if errno != 11:
                             print errstr
                         break
+                    self.time_last_update = time.time()
                     if read.endswith("\n"):
                         statusl = string.split(read, ":")
-                        if len(statusl) < 2:
+                        if len(statusl) < 3:
                             print "got garbage from dpkg: '%s'" % read
                             read = ""
                             break
                         status = statusl[2].strip()
                         #print status
                         if status == "error" or status == "conffile-prompt":
-                            self.expander.set_expanded(True)
+                            self.term_expander.set_expanded(True)
                         read = ""
                 self.progress.pulse()
                 while gtk.events_pending():
                     gtk.main_iteration()
                 time.sleep(0.2)
+                # if the terminal has not reacted for some time, do something
+                if (not self.term_expander.get_expanded() and 
+                    (self.time_last_update + GDEBI_TERMINAL_TIMEOUT) < time.time()):
+                  self.term_expander.set_expanded(True)
             self.progress.set_fraction(1.0)
-            
+            reaper.disconnect(signal_id)
     
     class InstallProgressAdapter(InstallProgress):
         def __init__(self,progress,term,label,term_expander):
@@ -597,6 +604,7 @@ class GDebi(SimpleGladeApp):
             self.term_expander = term_expander
             self.finished = False
             self.action = label
+            self.time_last_update = time.time()
             reaper = vte.reaper_get()
             reaper.connect("child-exited",self.child_exited)
             self.env = ["VTE_PTY_KEEP_FD=%s"% self.writefd,
@@ -617,13 +625,18 @@ class GDebi(SimpleGladeApp):
             apt_pkg.PkgSystemUnLock()
             self.action.set_markup("<i>"+_("Installing dependencies...")+"</i>")
             self.progress.set_fraction(0.0)
+            self.progress.set_text("")
         def statusChange(self, pkg, percent, status):
             self.progress.set_fraction(percent/100.0)
             self.progress.set_text(status)
+            self.time_last_update = time.time()
         def updateInterface(self):
             InstallProgress.updateInterface(self)
             while gtk.events_pending():
                 gtk.main_iteration()
+            if (not self.term_expander.get_expanded() and 
+                (self.time_last_update + GDEBI_TERMINAL_TIMEOUT) < time.time()):
+              self.term_expander.set_expanded(True)
         def fork(self):
             return self.term.forkpty(envv=self.env)
         def waitChild(self):
@@ -645,7 +658,11 @@ class GDebi(SimpleGladeApp):
             #print "stop()"
             pass
         def pulse(self):
-            self.progress.set_text(_("File %s of %s at %s/s" % (self.currentItems,self.totalItems,apt_pkg.SizeToStr(self.currentCPS))))
+            at_item = min(self.currentItems + 1, self.totalItems)
+            if self.currentCPS > 0:
+                self.progress.set_text(_("File %s of %s at %sB/s" % (at_item,self.totalItems,apt_pkg.SizeToStr(self.currentCPS))))
+            else:
+                self.progress.set_text(_("File %s of %s" % (at_item,self.totalItems)))
             self.progress.set_fraction(self.currentBytes/self.totalBytes)
             while gtk.events_pending():
                 gtk.main_iteration()
